#!/bin/bash
# Copyright (c) 2005-2011 Userful Corporation.  All rights reserved.
#
#
#
#
#
logfile="/var/log/userful-start.log"
CONFIG_FILE="/etc/usbaudio"
INPUT_CONFIG_FILE="/etc/usbinput"
ProgramName=`basename $0`
TEXTDOMAINDIR=/usr/share/locale
TEXTDOMAIN=userful-multimedia
function gettext
{
local RET=`su $2 -c "gettext \"$1\""`
echo "$RET"
}
source /opt/userful/bin/userful-bash-lib.sh
if [ ! -e /etc/X11/stations ]; then
echo "`date +"%b %d %T"` $ProgramName: /etc/X11/stations file is missing" >> $logfile
exit 0
fi
if [ ! -e $CONFIG_FILE ]; then
touch $CONFIG_FILE
fi
awk '{if (NF == 5) print $0}' $CONFIG_FILE > $CONFIG_FILE.old
rm -f $CONFIG_FILE
touch $CONFIG_FILE
chmod a+r $CONFIG_FILE
if [ ! -d /sys/class/sound ]; then
echo "`date +"%b %d %T"` $ProgramName: No audio device found." >> $logfile
exit 0
fi
AUDIO_UDIS=`hal-find-by-property --key "alsa.type" --string "playback"`
for UDI in $AUDIO_UDIS
do
ALSA_CARD=`hal-get-property --udi "$UDI" --key "alsa.card"`
ALSA_DEVICE=`hal-get-property --udi "$UDI" --key "alsa.device"`
if [[ "$ALSA_DEVICE" != "0" ]] ; then
continue
fi
PARENT_UDI=`hal-get-property --udi "$UDI" --key "info.parent"`
PARENT_SUBSYSTEM=`hal-get-property --udi "$PARENT_UDI" --key "linux.subsystem"`
if [ "$PARENT_SUBSYSTEM" == "sound" ]; then
DEV_VENDOR=`hal-get-property --udi "$PARENT_UDI" --key "sound.card_id"`
elif [ "$PARENT_SUBSYSTEM" == "usb" ]; then
DEV_VENDOR=`hal-get-property --udi "$PARENT_UDI" --key "usb.vendor"`
elif [ "$PARENT_SUBSYSTEM" == "pci" ]; then
DEV_VENDOR=`hal-get-property --udi "$PARENT_UDI" --key "pci.vendor"`
else
DEV_VENDOR=`hal-get-property --udi "$PARENT_UDI" --key "info.vendor"`
fi
if [ "$DEV_VENDOR" == "" ]; then
DEV_VENDOR="Unkown"
else # remove blank
DEV_VENDOR=`echo $DEV_VENDOR | tr "[[:blank:]]" "_"`
fi

DEV_PATH=`hal-get-property --udi "$PARENT_UDI" --key "linux.sysfs_path"`
#
PRE_ASSIGNED_SCREEN=`grep -m 1 $DEV_PATH $CONFIG_FILE.old | awk '{print $3}'`
PRE_ASSIGNED_FLAG=`grep -m 1 $DEV_PATH $CONFIG_FILE.old | awk '{print $4}'`
if [ "$PRE_ASSIGNED_SCREEN" != "" -a "$PRE_ASSIGNED_FLAG" == "W" ] ; then
echo "`date +"%b %d %T"` $ProgramName: Found $DEV_VENDOR($PARENT_SUBSYSTEM) on $CONFIG_FILE."  >> $logfile
ASSIGNED_SCREEN="$PRE_ASSIGNED_SCREEN"
ASSIGNED_FLAG="W"
if [ "${ASSIGNED_SCREEN:0:4}" == "Srn:" ]; then
SCREEN_NUM=`echo $ASSIGNED_SCREEN | awk -F':' '{print $2}' | awk -F'.' '{print $1}'`
else
SCREEN_NUM="-1"
fi
else
if [ "$PARENT_SUBSYSTEM" == "sound" ]; then
TMP_UDI=`hal-get-property --udi "$PARENT_UDI" --key "info.parent"`
TMP_SUBSYSTEM=`hal-get-property --udi "$TMP_UDI" --key "linux.subsystem"`
fi
if [ "$PARENT_SUBSYSTEM" == "usb" -o "$TMP_SUBSYSTEM" == "usb" ]; then
SCREEN_NUM=`/opt/userful/bin/get-screen-num-from-devpath -T 12 -P $DEV_PATH 2>/dev/null`
if [ "$SCREEN_NUM" == "-1" ]; then
echo "`date +"%b %d %T"` $ProgramName: Fails to auto-assign hw:$ALSA_CARD. Disabled." >> $logfile
ASSIGNED_SCREEN="Disabled"
else
ASSIGNED_SCREEN="Srn:$SCREEN_NUM.0"
fi
else # disable internal audio device by default.
ASSIGNED_SCREEN="Disabled"
fi
ASSIGNED_FLAG="R/W"
fi
if [ "$ASSIGNED_SCREEN" != "Disabled" ]; then
CONFLICT=`grep -c $ASSIGNED_SCREEN $CONFIG_FILE`
if [ $CONFLICT -gt 0 ]; then
echo "`date +"%b %d %T"` $ProgramName: Another audio device already assigned screen $ASSIGNED_SCREEN" >> $logfile
sed -i -e "/$ASSIGNED_SCREEN/d" $CONFIG_FILE
fi
if [ -f /etc/stations ] && [ -f /etc/startat ]; then
STARTAT=`cat /etc/startat`
REAL_STATION_NUM=`expr $SCREEN_NUM + $STARTAT`
if [ -e /opt/userful/locks/username$REAL_STATION_NUM ]; then
echo 1 > /opt/userful/locks/audio$REAL_STATION_NUM
fi
fi
fi
if [ "$PARENT_SUBSYSTEM" == "sound" ]; then
PARENT_SUBSYSTEM="$TMP_SUBSYSTEM"
fi
log "Assign audio device %s(hw:%s) to %s" "$DEV_VENDOR" "$ALSA_CARD" "$ASSIGNED_SCREEN"
if [ "$ASSIGNED_SCREEN" != "Disabled" ]; then
USER_NAME=""
if [ -e /opt/userful/locks/user.screen$SCREEN_NUM ]; then
USER_NAME=`cat /opt/userful/locks/user.screen$SCREEN_NUM`
fi
if [ -n "$USER_NAME" ]; then
if which pulseaudio > /dev/null 2>&1; then
su -c "/opt/userful/bin/multistation-audio-gen-conf.sh $ALSA_CARD pulse" $USER_NAME 
else
su -c "/opt/userful/bin/multistation-audio-gen-conf.sh $ALSA_CARD alsa" $USER_NAME 
fi
if [ "$ALSA_CARD" == "0" ]; then
/opt/userful/bin/multistation-audio-update-device.sh $USER_NAME
fi
/opt/userful/bin/multistation-audio-update-device.sh $USER_NAME $ALSA_CARD
fi
fi
echo "$DEV_VENDOR($PARENT_SUBSYSTEM) hw:$ALSA_CARD $ASSIGNED_SCREEN $ASSIGNED_FLAG $DEV_PATH" >> $CONFIG_FILE
done
while read DISABLED_DEVICE 
do
PRE_ASSIGNED_FLAG=`echo $DISABLED_DEVICE | awk '{print $4}'`

if [[ "$PRE_ASSIGNED_FLAG" == "W" ]]; then
continue
fi
MAX_STATION=`cat /etc/X11/stations`
for (( i=0; i < $MAX_STATION; i++ )); do
if  grep "Srn:$i.0" $CONFIG_FILE > /dev/null 2>&1 ; then
continue
fi
if [ $i -ge 10 ]; then
KBD_DEV=`grep "Srn#=$i" $INPUT_CONFIG_FILE | grep "(kbd)"`
else
KBD_DEV=`grep "Srn#=0$i" $INPUT_CONFIG_FILE | grep "(kbd)"`
fi
if [ "${KBD_DEV:1:1}" != "1" ]; then
continue
fi

ALSA_NAME=`echo $DISABLED_DEVICE | awk '{print $1}'`
ALSA_NUMBER=`echo $DISABLED_DEVICE | awk '{print $2}'`
ALSA_PATH=`echo $DISABLED_DEVICE | awk '{print $5}'`
sed -i -e "/$ALSA_NUMBER/d" $CONFIG_FILE
echo "$ALSA_NAME $ALSA_NUMBER Srn:$i.0 R/W $ALSA_PATH" >> $CONFIG_FILE

ALSA_CARD=`echo $ALSA_NUMBER|awk -F ':' '{print $2}'`
USER_NAME=""
if [ -e /opt/userful/locks/user.screen$i ]; then
USER_NAME=`cat /opt/userful/locks/user.screen$i`
fi
if [ -n "$USER_NAME" ]; then
if which pulseaudio > /dev/null 2>&1; then
su -c "/opt/userful/bin/multistation-audio-gen-conf.sh $ALSA_CARD pulse" $USER_NAME 
else
su -c "/opt/userful/bin/multistation-audio-gen-conf.sh $ALSA_CARD alsa" $USER_NAME 
fi
if [ "$ALSA_CARD" == "0" ]; then
/opt/userful/bin/multistation-audio-update-device.sh $USER_NAME
fi
/opt/userful/bin/multistation-audio-update-device.sh $USER_NAME $ALSA_CARD
fi
break
done
done < <(grep "(pci)" $CONFIG_FILE)
#
